/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "callgrind.h"
#include <QStringList>
#include <valgrind/callgrind.h>

using namespace Rdfs;

bool Callgrind::exec(const QStringList& arguments)
{
    if (arguments.at(0) == "start") {
        CALLGRIND_START_INSTRUMENTATION;
        CALLGRIND_ZERO_STATS;
    }
    else {
        CALLGRIND_STOP_INSTRUMENTATION;
    }
    return true;
}

QString Callgrind::checkArguments(const QStringList& arguments) const
{
    if (arguments.at(0) != "start" && arguments.at(1) != "stop") {
        return tr("Action must be start or stop");
    }
    return QString();
}

bool Callgrind::requireConnection() const
{
    return false;
}

QList< Rdfs::CommandParameter > Callgrind::arguments() const
{
    Rdfs::CommandParameter arg;
    arg.name = tr("action");
    arg.help = tr("Whether to start or stop instrumentation");
    arg.examples << "start" << "stop";
    arg.required = true;
    return QList<CommandParameter>() << arg;
}

QStringList Callgrind::aliases() const
{
    return QStringList() << "cg";
}

QString Callgrind::description() const
{
    return tr("Start or stop callgrind instrumentation.");
}

QString Callgrind::name() const
{
    return "callgrind";
}

Callgrind::Callgrind(Rdfs::RdfsUpdate* parent): RdfsCommand(parent)
{

}

Callgrind::~Callgrind()
{

}

#include "callgrind.moc"