/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef RDFS_FINDQUERIES_H
#define RDFS_FINDQUERIES_H

#include <QSet>

#include "rdfschaincommand.h"
#include "../rdfsbackend.h"
#include "../triple.h"

namespace Rdfs {

class ChainDefinition {
public:
    explicit ChainDefinition(const QString &predicate = QString());
    ChainDefinition(const Triple &t);

    // Plain fields
    QString start;
    QString predicate;
    QString end;
    quint32 size;

    bool operator<(const ChainDefinition &b) const;
    const ChainDefinition operator+(const Rdfs::ChainDefinition& b) const;
    ChainDefinition& operator+=(const ChainDefinition &b);
};

class FindQueries : public Rdfs::RdfsChainCommand
{
Q_OBJECT
private:
    QList< Rdfs::Triple > generateClass1Queries(int n, int valid, Rdfs::RdfsTable table) const;
    QList< Rdfs::Triple > generateClass2Queries(int n, int valid, Rdfs::RdfsTable table) const;
    QList< Rdfs::Triple > generateClass3Queries(int n, int valid, Rdfs::RdfsTable table);
    QList< Rdfs::Triple > generateClass4Queries(int n, int valid, Rdfs::RdfsTable table);

    bool chance(float p) const;

    QList<ChainDefinition> collectChains(const QString& predicate, Rdfs::RdfsTable table, int maxAmount);
    QList< ChainDefinition > disconnectedChains(const QString& predicate, Rdfs::RdfsTable table) const;

    QList<ChainDefinition> m_traversedChains;
    int m_maxChains;

protected:
    virtual void foundChain(const QList< QString >& atoms, const QString& predicate);
    virtual void skipTriple(const Rdfs::Triple& t);
    virtual void startChain(const Rdfs::Triple& t);

public:
    virtual bool exec(const QStringList& arguments);
    virtual QString checkArguments(const QStringList& arguments) const;
    virtual bool requireConnection() const { return true; };
    virtual QList< Rdfs::CommandParameter > arguments() const;
    virtual QStringList aliases() const { return QStringList() << "fq"; };
    virtual QString description() const { return tr("Find interesting queries for a specific property or class."); };
    virtual QString name() const { return "FindQueries"; };

    FindQueries(Rdfs::RdfsUpdate* parent);
    virtual ~FindQueries();
};

}

#endif // RDFS_FINDQUERIES_H
