/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "rdfschaincommand.h"

#include <QStack>

#include "../rdfscursor.h"
#include "../rdfsupdate.h"

using namespace Rdfs;

RdfsChainCommand::RdfsChainCommand(Rdfs::RdfsUpdate* parent): RdfsCommand(parent)
{

}

void Rdfs::RdfsChainCommand::traverse(const QString& predicate, Rdfs::RdfsTable table, const QString &root)
{
    Triple search;
    search.subject = root;
    search.predicate = predicate;

    RdfsCursor *c = framework()->store()->cursor(search, table);
    if (!c->open()) {
        delete c;
        return;
    }

    while (!c->eof()) {
        Triple t = c->current();
        search.subject = QString();
        search.predicate = predicate;
        search.object = t.subject;

        if (framework()->store()->hasTriple(search, table)) {
            // Not a root item
            skipTriple(t);
            c->next();
            continue;
        }

        startChain(t);

        // Prepare stack
        QStack< QList<QString> > q;
        QList<QString> currentChain;
        currentChain.append(t.subject);
        currentChain.append(t.object);
        q.push(currentChain);

        // Prepare search triple - predicate is already set ok
        search.object = QString();

        while (!q.isEmpty()) {
            currentChain = q.pop();
            QString atom = currentChain.last();

            // Find triples going down
            search.subject = atom;
            QList<Triple> triples = framework()->store()->findTriples(search, table);

            if (triples.isEmpty()) {
                // Current chain is final
                foundChain(currentChain, predicate);
                continue;
            }

            foreach(Triple t, triples) {
                // Push new sub-chain on stack
                QList<QString> newChain(currentChain);
                newChain.append(t.object);
                q.push(newChain);
            }
        }

        c->next();
    }
    c->close();
    delete c;
}


RdfsChainCommand::~RdfsChainCommand()
{

}

