/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "rdfslistrules.h"
#include "../rdfsupdate.h"
#include "../rdfssql.h"
#include "../rdfslogger.h"

using namespace Rdfs;


RdfsListRules::RdfsListRules(RdfsUpdate* parent): RdfsCommand(parent) {

}

bool RdfsListRules::exec(const QStringList& arguments) {
    QList<Rule> rules = framework()->store()->rules();
    // Format separator
    const QString sep = " ";
    Q_FOREACH(Rule r, rules) {
        QString t1s, t2s;
        t1s = r.first.subject + sep + r.first.predicate + sep + r.first.object;
        t2s = r.second.subject + sep + r.second.predicate + sep + r.second.object;

        RdfsLogger::instance()->log(QString("%1 -> %2")
                                    .arg(t1s, 16)
                                    .arg(t2s));
    }
    return true;
}

QString RdfsListRules::checkArguments(const QStringList& arguments) const {
    // No arguments -- nothing to check
    return QString();
}

bool RdfsListRules::requireConnection() const {
    return true;
}

QList<CommandParameter> RdfsListRules::arguments() const {
    return QList<CommandParameter>();
}

QStringList RdfsListRules::aliases() const {
    return QStringList() << "lr";
}

QString RdfsListRules::description() const {
    return tr("List all defined deterministic tuple-generating dependencies. Use addRule and removeRule commands to modify these.");
}

QString RdfsListRules::name() const {
    return "listRules";
}


RdfsListRules::~RdfsListRules() {

}


#include "rdfslistrules.moc"
