/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "rdfsstats.h"

#include "../rdfsupdate.h"
#include "../rdfslogger.h"

using namespace Rdfs;

RdfsStats::RdfsStats(RdfsUpdate* parent): RdfsCommand(parent)
{

}

bool RdfsStats::exec(const QStringList& arguments)
{
    if (framework()->store()->capabilities() & RDFS_CACHESTATS) {
        print();
        return true;
    }
    return false;
}

QString RdfsStats::checkArguments(const QStringList& arguments) const
{
    if ((framework()->store()->capabilities() & RDFS_CACHESTATS) == 0) {
        return tr("The currently used back-end does not support this operation");
    }
    return QString();
}

void RdfsStats::print()
{
    print(framework()->store()->statistics());
}

void RdfsStats::print(const Rdfs::CacheStats& statistics)
{
    float hitPct, missPct;
    hitPct = (statistics.hits / (float)(statistics.hits + statistics.misses)) * 100.0;
    missPct = (statistics.misses / (float)(statistics.hits + statistics.misses)) * 100.0;
    RdfsLogger::instance()->log("=====");
    RdfsLogger::instance()->log(tr("Cache size: %L1 (%L2KiB)").arg(statistics.cachesize).arg(statistics.cachesize / 1024));
    RdfsLogger::instance()->log(tr("Page size: %L1 (%L2KiB)").arg(statistics.pagesize).arg(statistics.pagesize / 1024));
    RdfsLogger::instance()->log(tr("Cache hits: %L1 (%L2%)").arg(statistics.hits).arg(hitPct, 0, 'f', 2));
    RdfsLogger::instance()->log(tr("Cache misses: %L1 (%L2%)").arg(statistics.misses).arg(missPct, 0, 'f', 2));
    RdfsLogger::instance()->log(tr("Pages created in cache: %L1").arg(statistics.created));
    RdfsLogger::instance()->log(tr("Pages read into cache: %L1").arg(statistics.read));
    RdfsLogger::instance()->log(tr("Pages written from cache: %L1").arg(statistics.written));
    RdfsLogger::instance()->log("=====");
}


Rdfs::RdfsStats::~RdfsStats()
{

}

#include "rdfsstats.moc"
