/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "rdfstriplecommand.h"

#include "../rdfsupdate.h"

using namespace Rdfs;

RdfsTripleCommand::RdfsTripleCommand(RdfsUpdate* parent)
  : RdfsCommand(parent)
{
}

QString RdfsTripleCommand::checkArguments(const QStringList& arguments) const
{
    QString argumentsError;
    int base = 0;

    if (arguments.count() == 4) {
        if (arguments.at(3).compare("original", Qt::CaseInsensitive) != 0 &&
            (arguments.at(3).compare("update", Qt::CaseInsensitive) != 0)) {
            argumentsError = tr("Unknown table %1 given.").arg(arguments.at(3));
        }
        ++base;
    }
    // Argument count is checked by framework

    return argumentsError;
}

void RdfsTripleCommand::extractArguments(const QStringList& arguments, RdfsTable* table, Triple* t) const
{
    t->subject =   (arguments.at(0) == "_") ? "" : arguments.at(0);
    t->predicate = (arguments.at(1) == "_") ? "" : arguments.at(1);
    t->object =    (arguments.at(2) == "_") ? "" : arguments.at(2);

    if (arguments.count() == 4) {
        if (arguments.at(3).compare("original", Qt::CaseInsensitive) == 0) {
            (*table) = RDFS_ORIGINAL;
        }
        else if (arguments.at(3).compare("update", Qt::CaseInsensitive) == 0) {
            (*table) = RDFS_UPDATE;
        }
    }
    else {
        (*table) = RDFS_ORIGINAL;
    }
}

QList<Rdfs::CommandParameter> RdfsTripleCommand::arguments() const
{
    CommandParameter subject;
    subject.name = tr("subject");
    subject.examples << tr("s");
    subject.help = tr("The subject of the RDFS Triple (\"_\" to keep blank)");
    subject.required = true;

    CommandParameter predicate;
    predicate.name = tr("predicate");
    predicate.examples << tr("p");
    predicate.help = tr("The predicate of the RDFS Triple (\"_\" to keep blank)");
    predicate.required = true;

    CommandParameter object;
    object.name = tr("object");
    object.examples << tr("o");
    object.help = tr("The object of the RDFS Triple (\"_\" to keep blank)");
    object.required = true;

    CommandParameter table;
    table.name = tr("table");
    table.examples << "original";
    table.examples << "update";
    table.help = tr("Table to use for this command");
    table.required = false;

    return (QList<CommandParameter>() << subject << predicate << object << table);
}

RdfsTripleCommand::~RdfsTripleCommand()
{

}

#include "rdfstriplecommand.moc"
