/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "rdfsbackend.h"
#include "rdfslistcursor.h"

using namespace Rdfs;

RdfsBackend::RdfsBackend()
{

}

RdfsBackend::~RdfsBackend()
{

}

QString RdfsBackend::tableName(RdfsTable t) {
    switch(t) {
        case RDFS_ORIGINAL:
        default:
            return "rdfs_original";
            break;
        case RDFS_UPDATE:
            return "rdfs_update";
            break;
        case RDFS_RULE:
            return "rdfs_rule_triple";
            break;
    }
}

void RdfsBackend::insertRule(Triple t1, Triple t2) {
    insertRule(Rule(t1, t2));
}

void RdfsBackend::removeRule(Triple t1, Triple t2) {
    removeRule(Rule(t1, t2));
}

Rdfs::RdfsCursor* Rdfs::RdfsBackend::cursor(Triple q, RdfsTable table)
{
    RdfsListCursor *m = new RdfsListCursor(this, q, table);
    return m;
}

bool Rdfs::RdfsBackend::hasTriple(const Rdfs::Triple& t, RdfsTable table)
{
    bool result = false;
    RdfsCursor *c = cursor(t, table);
    if (c != 0) {
        if (c->open()) {
            result = true;
        }
        delete c;
    }
    return result;
}

Rdfs::CacheStats Rdfs::RdfsBackend::statistics() const
{
    CacheStats s;
    s.cachesize = 0;
    s.pagesize = 0;
    s.hits = 0;
    s.misses = 0;
    s.created = 0;
    s.read = 0;
    s.written = 0;
    return s;

}
