/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef RDFS_RDFSBDBBT_H
#define RDFS_RDFSBDBBT_H

#include "rdfsbackend.h"

#include <db.h>


namespace Rdfs {

class RdfsBdbBt : public Rdfs::RdfsBackend
{
private:
    friend class RdfsBdbBtCursor;

    DB_ENV *m_db_env;
    DB *m_db_original[3];
    DB *m_db_update[3];
    DB *m_db_rule[3];

    enum INDEX_ORDER {
        SOP = 0,
        PSO = 1,
        OPS = 2
    };

    DB **dbs(Rdfs::RdfsTable table);

    static DBT tripleToDbt(Rdfs::Triple t, Rdfs::RdfsBdbBt::INDEX_ORDER order);
    static Rdfs::Triple dbtToTriple(DBT* data, Rdfs::RdfsBdbBt::INDEX_ORDER order);

    int sizeStringToBytes(const QString &sizeString);
    virtual void setPageSize(const QString &size);
    virtual void setPageSize(int bytes);
    virtual void setCacheSize(const QString &size);
    virtual void setCacheSize(int bytes);

public:
    virtual void truncate(Rdfs::RdfsTable table = RDFS_ORIGINAL);
    virtual void removeRule(Rdfs::Rule r);
    virtual QList< QPair< Rdfs::Triple, Rdfs::Triple > > rules();
    virtual void insertRule(Rdfs::Rule r);
    virtual void removeTriple(const Rdfs::Triple& t, Rdfs::RdfsTable table = RDFS_ORIGINAL);
    virtual QList< Rdfs::Triple > findTriples(const Rdfs::Triple& t, Rdfs::RdfsTable table);
    virtual int checkTriple(const Rdfs::Triple& t, Rdfs::RdfsTable table = RDFS_ORIGINAL);
    virtual void insertTriple(const Rdfs::Triple& t, Rdfs::RdfsTable table = RDFS_ORIGINAL);
    virtual int capabilities() const;
    virtual void close();
    virtual bool isOpen();
    virtual void startBulkLoad();
    virtual void endBulkLoad();
    virtual bool hasTriple(const Rdfs::Triple& t, Rdfs::RdfsTable table = RDFS_ORIGINAL);
    virtual Rdfs::RdfsCursor* cursor(Rdfs::Triple q, Rdfs::RdfsTable table = RDFS_ORIGINAL);
    virtual Rdfs::CacheStats statistics() const;

    RdfsBdbBt(const QString& dir, const QString& file, const QString& updateFile, const QString& rulesFile,
                   const QString& cacheSize = QString(), const QString& pageSize = QString());
    virtual ~RdfsBdbBt();
};

}

#endif // RDFS_RDFSBDBBT_H
