/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef RDFSCOMMAND_H
#define RDFSCOMMAND_H

#include <QObject>
#include <QString>
#include <QStringList>
#include <QList>

namespace Rdfs {
class RdfsUpdate;
class RdfsSql;

struct CommandParameter {
    QString name;
    QString help;
    bool required;
    QStringList examples;
};

class RdfsCommand : public QObject {
Q_OBJECT;
private:
    RdfsUpdate *m_rdfsUpdate;

protected:
    inline RdfsUpdate *framework() const { return m_rdfsUpdate; };

public:
    RdfsCommand(RdfsUpdate *parent);

    virtual QString name() const = 0;
    virtual QString description() const = 0;
    virtual QStringList aliases() const = 0;
    virtual QList<CommandParameter> arguments() const = 0;
    virtual bool requireConnection() const = 0;

    virtual QString checkArguments(const QStringList &arguments) const = 0;
    virtual bool exec(const QStringList &arguments) = 0;

    virtual ~RdfsCommand();

};
};
#endif // RDFSCOMMAND_H
