/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef RDFS_SUBCLASSRULE_H
#define RDFS_SUBCLASSRULE_H

#include "transitiverule.h"

namespace Rdfs {

/**
 * Rule which checks the Subclass rule:
 * (A,sc,B) & (B,sc,C) => (A,sc,C)
 *
 * Also checks reflexivity rule, if all the mappings have been added
 * (A,sc,B) => (A,sc,A) & (B,sc,B)
 * (A,p,X) => (A,sp,A) for p in { map(dom), map(range), map(type) }
 *
 * This corrensponds to step 3a and 3b of the algorithm.
 */

class SubclassRule : public Rdfs::TransitiveRule
{

public:
    SubclassRule();
    virtual ~SubclassRule();

    virtual bool check(Rdfs::Triple t, Rdfs::RdfsTable table, RdfsBackend *store);
};

}

#endif // RDFS_SUBCLASSRULE_H
