/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef RDFS_SUBPROPERTYRULE_H
#define RDFS_SUBPROPERTYRULE_H

#include "transitiverule.h"

namespace Rdfs {

/**
 * Rule which checks the Subproperty rule:
 * (A,sp,B) & (B,sp,C) => (A,sp,C)
 *
 * Also checks reflexivity rule, if all the mappings have been added
 * (X,A,Y) => (A,sp,A)
 * (p,sp,p) if p in mappings
 * (A,sp,B) => (A,sp,A) & (B,sp,B)
 * (A,p,X) => (A,sp,A) for p in { map(dom), map(range) }
 *
 * This corresponds to step 2a, 2b and 4 of the algorithm.
 */
class SubpropertyRule : public TransitiveRule
{
private:
    /**
     * Check reflexivity of sp rule (step 2b)
     */
    bool checkReflexive(Rdfs::Triple t, Rdfs::RdfsTable table, RdfsBackend *store);

    /**
     * Check for triple if t.predicate not in rhodf (step 4 of the algorithm)
     */
    bool checkNonRhodf(Rdfs::Triple t, Rdfs::RdfsTable table, RdfsBackend *store);

public:
    SubpropertyRule();
    virtual ~SubpropertyRule();

    virtual bool check(Rdfs::Triple t, Rdfs::RdfsTable table, RdfsBackend *store);
};

}

#endif // RDFS_SUBPROPERTYRULE_H
