/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <QStringList>

#include "rdfssqlite.h"
#include <QSqlRecord>
#include <QSqlQuery>
#include <QSqlField>

using namespace Rdfs;

RdfsSqlite::RdfsSqlite(const QString& fileName)
    : RdfsSql("QSQLITE")
{
    setDatabaseName(fileName);
    if (open()) {
        initializeTables();
    }
}

void RdfsSqlite::initializeRdfsOriginal() {
    if (!databaseTables().contains("rdfs_original")) {
        QSqlQuery query = QSqlQuery(*this);
        query.prepare("CREATE TABLE rdfs_original ("
            "id        INTEGER PRIMARY KEY AUTOINCREMENT, "
            "subject   TEXT NOT NULL, "
            "predicate TEXT NOT NULL, "
            "object    TEXT NOT NULL, "
            "UNIQUE (subject, predicate, object))"
        );
        executeQuery(&query);
    }
}

void RdfsSqlite::initializeRdfsRules() {
    if (!databaseTables().contains("rdfs_rule_triple")) {
        QSqlQuery query = QSqlQuery(*this);
        query.prepare("CREATE TABLE rdfs_rule_triple ("
            "id        INTEGER PRIMARY KEY AUTOINCREMENT, "
            "subject   TEXT NOT NULL, "
            "predicate TEXT NOT NULL, "
            "object    TEXT NOT NULL, "
            "UNIQUE (subject, predicate, object))"
            );
        executeQuery(&query);
    }

    if (!databaseTables().contains("rdfs_rule")) {
        QSqlQuery query = QSqlQuery(*this);
        query.prepare("CREATE TABLE rdfs_rule ("
            "source    INTEGER, "
            "target    INTEGER, "
            "PRIMARY KEY (source, target))");
        executeQuery(&query);
    }
}


void RdfsSqlite::initializeRdfsUpdate() {
    if (!databaseTables().contains("rdfs_update")) {
        QSqlQuery query = QSqlQuery(*this);
        query.prepare("CREATE TABLE rdfs_update ("
            "id        INTEGER PRIMARY KEY AUTOINCREMENT, "
            "subject   TEXT NOT NULL, "
            "predicate TEXT NOT NULL, "
            "object    TEXT NOT NULL, "
            "UNIQUE (subject, predicate, object))"
        );
        executeQuery(&query);
    }
}


void RdfsSqlite::insertTriple (const Rdfs::Triple& t, RdfsTable table)
{
    // Triple has to be valid
    if (t.subject.isEmpty() || t.predicate.isEmpty() || t.object.isEmpty()) {
        return;
    }

    // Variables
    QString tableStr = tableName(table);

    // Start transaction
    QSqlQuery query(*this);

    // Insert record
    query.prepare("INSERT INTO " + tableStr + " (subject, predicate, object) "
                  "VALUES (?, ?, ?)");
    query.bindValue(0, t.subject);
    query.bindValue(1, t.predicate);
    query.bindValue(2, t.object);
    executeQuery(&query);
}

void RdfsSqlite::truncate (RdfsTable table)
{
    // SQlite doesn't support "TRUNCATE TABLE" -> use DELETE FROM
    QString tableStr = tableName(table);

    QSqlQuery query(*this);
    query.prepare("DELETE FROM " + tableStr);
    executeQuery(&query);
}

RdfsSqlite::~RdfsSqlite() {
    // Nothing special to clean up
}
