/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "rdfsvirtuoso.h"

#include <Soprano/Backend>
#include <Soprano/BackendSettings>
#include <Soprano/Soprano>

using namespace Rdfs;

RdfsVirtuoso::RdfsVirtuoso(const QString &path)
    : RdfsSoprano()
{
    m_backend = Soprano::discoverBackendByName("virtuosobackend");

    Soprano::BackendSettings settings;
    settings << Soprano::BackendSetting(Soprano::BackendOptionStorageDir, path);

    setModel(m_backend->createModel(settings));
}

RdfsVirtuoso::RdfsVirtuoso(const QString& host, int port, const QString& username, const QString& password)
    : RdfsSoprano()
{
    m_backend = Soprano::discoverBackendByName("virtuosobackend");

    Soprano::BackendSettings settings;
    settings << Soprano::BackendSetting(Soprano::BackendOptionHost, host);
    settings << Soprano::BackendSetting(Soprano::BackendOptionPort, port);
    settings << Soprano::BackendSetting(Soprano::BackendOptionUsername, username);
    settings << Soprano::BackendSetting(Soprano::BackendOptionPassword, password);

    setModel(m_backend->createModel(settings));
}

int RdfsVirtuoso::capabilities() const
{
    return RDFS_INFER_SP | RDFS_INFER_SC;
}


RdfsVirtuoso::~RdfsVirtuoso()
{

}
