/*
    RdfsUpdate: A research tool for RDFS data
    Copyright (C) 2009-2011 Eindhoven University of Technology - www.tue.nl
    Copyright (C) 2009-2011 Wouter Haffmans <wouter@simply-life.net>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef TRIPLE_H
#define TRIPLE_H

#include <QString>
#include <QHash>
#include <QPair>

namespace Rdfs {
struct Triple {
public:
    // Constructor: set default distance to 0
    Triple() {
        this->distance = 0;
    }

    QString subject;
    QString predicate;
    QString object;
    unsigned char distance;
};

typedef QPair<Triple, Triple> Rule;

inline uint qHash(const Rdfs::Triple &t) {
    // XOR values of the triple
    return (::qHash(t.subject) ^ ::qHash(t.predicate) ^ ::qHash(t.object));
}
}


// Provide operator<() for Triples, as they are used in sets, maps and hashes
/**
 * Check if t1 is smaller than t2. This compares (in order) subject, predicate
 * and object. I.e., if the subjects are not equal, this results t1.subject < t2.subject.
 * Otherwise, if the predicates are not equal, this results t1.predicate < t2.predicate.
 * If subject and predicate are equal, then t1.object < t2.object is returned
 */
inline bool operator<(const Rdfs::Triple &t1, const Rdfs::Triple &t2) {
    if (t1.subject != t2.subject) {
        return (t1.subject < t2.subject);
    }
    else if (t1.predicate != t2.predicate) {
        return (t1.predicate < t2.predicate);
    }
    else {
        return (t1.object < t2.object);
    }
}

inline bool operator==(const Rdfs::Triple &t1, const Rdfs::Triple &t2) {
    return (t1.subject == t2.subject &&
            t1.predicate == t2.predicate &&
            t1.object == t2.object);
}

#endif
